import logging
import os
from typing import Text, Union

import requests

from . import ProviderBase

log = logging.getLogger(__name__)


class Cloudflare(ProviderBase):
    """
    Cloudflare provider
    """

    def __init__(
        self, api_key: Text = None, email_address: Text = None, account_id: Text = None, hugo_version: Text = None
    ):
        """

        :param hugo_version: New Hugo version
        :param api_key: Authentication key for Cloudflare
        :param email_address: Registered email address
        :param account_id: Cloudflare worker account ID
        :raises ValueError: If ``email_address`` and ``account_id`` is not provided
        """
        super().__init__(api_key, None, hugo_version)

        self.account_id = account_id

        if not email_address:
            email_address = os.environ.get("CLOUDFLARE_EMAIL", None)

        if not self.account_id:
            self.account_id = os.environ.get("CLOUDFLARE_WORKER_ACCOUNT_ID", None)

        if not email_address and not self.account_id and not self.hugo_version:
            raise ValueError("email_address, account_id or hugo_version not provided")

        if email_address:
            self.headers = {"X-Auth-Email": email_address, "X-Auth-Key": self.api_key}
        else:
            self.headers = {"Authorization": f"Bearer {self.api_key}"}

    def update_api(self, project_name: Text) -> requests.Response:
        """
        Updates Cloudflare Pages environment variable of ``HUGO_VERSION``.

        :param project_name: Name of the project to update
        :return:
        """

        data = {
            "deployment_configs": {
                "preview": {"env_vars": {"delete_this_env_var": None, "HUGO_VERSION": {"value": self.hugo_version}}},
                "production": {"env_vars": {"delete_this_env_var": None, "HUGO_VERSION": {"value": self.hugo_version}}},
            }
        }

        response = requests.patch(
            f"https://api.cloudflare.com/client/v4/accounts/{self.account_id}/pages/projects/{project_name}",
            json=data,
            headers=self.headers,
        )
        return response

    def get_projects(self, project_name: Text = None) -> requests.Response:
        """
        This checks and gets the projects available

        :param project_name: Name of the project to get
        :return: Returns HTTP response
        """

        if not project_name:
            response = requests.get(
                f"https://api.cloudflare.com/client/v4/accounts/{self.account_id}/pages/projects", headers=self.headers
            )
        else:
            response = requests.get(
                f"https://api.cloudflare.com/client/v4/accounts/{self.account_id}/pages/projects/{project_name}",
                headers=self.headers,
            )
        return response

    def current_version(self, project_name: Text = None) -> Union[str, None]:
        """
        Gets the current Hugo version

        :param project_name: Name of the project to check from
        """
        response = requests.get(
            f"https://api.cloudflare.com/client/v4/accounts/{self.account_id}/pages/projects/{project_name}",
            headers=self.headers,
        )

        data = response.json()
        try:
            return data["result"]["deployment_configs"]["production"]["env_vars"]["HUGO_VERSION"]["value"]
        except KeyError as e:
            log.debug(e)
            return None



import json
import logging
import platform
from pathlib import Path

import requests
from rich import print

log = logging.getLogger(__name__)

if platform.system() == "Windows":
    from .terminal_commands.windows import *  # noqa
elif platform.system() == "Linux" or platform.system() == "Darwin":
    from .terminal_commands.posix import *  # noqa
else:
    raise OSError("Unknown OS")


def bin_folder() -> str:
    """
    Gives the path of the user bin folder if exists else a bin folder is created in the
    ``<user home>/bin``

    :return: ``bin`` location
    """

    bin_path = Path(Path.home(), "bin")

    if not bin_path.is_dir():
        log.debug(f"bin directory does not exists. Creating one now. New path: {bin_path!r}")
        bin_path.mkdir()

    return str(bin_path)


def get_latest_version_api(override_version: str = None) -> str:
    """
    Get the latest Hugo version

    :param override_version: An override version of Hugo
    :return: version number
    """

    if override_version is not None:
        hugo_response = requests.get(f"https://api.github.com/repos/gohugoio/hugo/releases/tags/v{override_version}")
        if hugo_response.ok:
            return override_version
        else:
            log.debug("Override version request error occurred", hugo_response.content)
            print(
                f"\n[red bold]Hugo v{override_version} does not exists. See https://github.com/gohugoio/hugo/releases"
                " for more information."
            )
            exit(1)

    hugo_response = requests.get("https://api.github.com/repos/gohugoio/hugo/releases/latest")
    hugo_response = json.loads(hugo_response.content.decode("utf-8"))["tag_name"][1:]

    return hugo_response



from typing import Text

from . import ProviderBase


class Netlify(ProviderBase):
    """
    Netlify provider
    """

    def update_config_file(self, file_name: Text):
        """
        Updates ``netlify.yaml`` file with Hugo's version

        :param file_name: Path of ``netlify.yaml``
        :return:
        """
        pass



import subprocess

from packaging import version

from . import Hugo, hugo_version_cmd

__all__ = ["check_hugo"]


def check_hugo() -> Hugo:
    """
    Checks for Hugo instillation

    :return Hugo: A dataclass object with ``path``, ``exists`` and ``version``
    """

    try:
        _path = subprocess.check_output(["which", "hugo"]).strip()
    except Exception:
        return Hugo("", False, version.Version("0"))

    _hugo_version_str = hugo_version_cmd()

    try:
        _hugo_version = _hugo_version_str.decode("utf-8").split(" ")[4].split("/")[0].split("-")[0]
        ver = version.Version(_hugo_version)
    except version.InvalidVersion:
        _hugo_version = _hugo_version_str.decode("utf-8").split(" ")[1].split("-")[0]
        ver = version.Version(_hugo_version)

    return Hugo(_path.decode("utf-8"), True, ver)



from typing import List


def humanise_list(items: List[str]) -> str:
    """
    Humanise a list of items

    :param items: List of items
    :return: Humanised string
    """

    if len(items) == 1:
        return items[0]
    elif len(items) == 2:
        return f"{items[0]} and {items[1]}"
    else:
        return ", ".join(items[:-1]) + f" and {items[-1]}"



import logging
import platform
import tarfile
import zipfile

from .checks import bin_folder

log = logging.getLogger(__name__)


def install_hugo(extract_from: str, os_type: str = platform.system(), move_to: str = bin_folder()) -> str:
    """
    Installs the ``hugo`` binary at a given path

    :param extract_from: The path zip/tar file downloaded
    :param os_type: Type of OS - ``Windows``, ``Darwin`` or ``posix``.
    :param move_to: Instillation location. Defaults to :py:func:`bin_folder`
    :return: Installation path
    """

    if os_type == "Darwin" or os_type == "Linux":
        with tarfile.open(extract_from, "r:gz") as f:
            f.extract("hugo", move_to)
    elif os_type == "Windows":
        with zipfile.ZipFile(extract_from, "r") as f:
            f.extract("hugo.exe", move_to)

    return move_to



import json
import logging
import os
from typing import List

import toml
from rich.console import Console
from rich.prompt import Prompt

from uhugo.post_install.detect_providers import Provider

log = logging.getLogger(__name__)


class UpdateProvider:
    """
    Update providers with Hugo version
    """

    def __init__(self, providers: List[Provider], console: Console, version: str):
        """
        :param providers: List of Provider objects
        :param console: Rich Console object
        :param version: Hugo version
        """
        self.providers = providers
        self.console = console
        self.version = version

    def update(self):
        """
        Update providers with Hugo version
        """
        _providers = {
            "netlify": True,
            "vercel": True,
            "cloudflare": True,
        }
        for provider in self.providers:
            if provider.name == "netlify":
                _providers[provider.name] = self.update_netlify(provider)
            elif provider.name == "vercel":
                _providers[provider.name] = self.update_vercel(provider)
            elif provider.name == "cloudflare":
                _providers[provider.name] = self.update_cloudflare(provider)

        failed_providers = _providers.items()

        if len(failed_providers) > 0:
            for provider, status in _providers.items():
                if not status:
                    self.console.print(f"\n{provider} provider(s) not updated to {self.version} :x:", style="red bold")
            return
        self.console.print(f"\nAll providers updated to {self.version} :tada:", style="green bold")

    def update_netlify(self, provider: Provider) -> bool:
        """
        Update Netlify provider with a latest Hugo variables

        :param provider: Provider object
        :return: False if not updated
        """
        try:
            with open(provider.path, "r+") as f:
                data = toml.load(f)
                if data["context"]["production"]["environment"]["HUGO_VERSION"] == self.version:
                    self.console.print(
                        ":heavy_check_mark: Netlify - Hugo version is already up to date", style="green bold"
                    )
                    return True
                data["context"]["production"]["environment"]["HUGO_VERSION"] = self.version
                data["context"]["deploy-preview"]["environment"]["HUGO_VERSION"] = self.version
                f.seek(0)
                toml.dump(data, f)
                f.truncate()
            self.console.print(":heavy_check_mark: Netlify", style="green bold")
        except FileNotFoundError as e:
            log.debug(e)
            self.console.print(f":x: Netlify - File '{provider.path}' not found", style="bold red")
            return False

    def update_vercel(self, provider: Provider) -> bool:
        """
        Update Vercel provider with a latest Hugo variables

        :param provider: Provider object
        :return: False if not updated
        """

        try:
            with open(provider.path, "r+") as f:
                json_data = json.load(f)
                if json_data["build"]["env"]["HUGO_VERSION"] == self.version:
                    self.console.print(
                        ":heavy_check_mark: Vercel - Hugo version is already up to date", style="green bold"
                    )
                    return True
                json_data["build"]["env"]["HUGO_VERSION"] = self.version
                f.seek(0)
                json.dump(json_data, f, indent=4)
                f.truncate()
            self.console.print(":heavy_check_mark: Vercel", style="green bold")
        except FileNotFoundError as e:
            log.debug(e)
            self.console.print(f":x: Vercel - File '{provider.path}' not found", style="bold red")
            return False

    def update_cloudflare(self, provider: Provider) -> bool:
        """
        Update Cloudflare provider with a latest Hugo variables

        :param provider: Provider object
        :return: False if not updated
        """

        # Get the environment variables from the config file
        for key, val in provider.model_dump().items():
            if val and val.startswith("env"):
                try:
                    setattr(provider, key, os.environ[val.split(":")[1]])
                except KeyError as e:
                    log.debug(e)
                    self.console.print(f"Environment variable '{val.split(':')[1]}' not found", style="bold red")
                    return False

        from ..post_install.providers.cloudflare import Cloudflare

        cf = Cloudflare(provider.api_key, provider.email_address, provider.account_id, self.version)
        projects = cf.get_projects(provider.project).json()
        # TODO: Stop spinner before continuing or else this will not show up
        if projects["success"] and isinstance(projects["result"], list):
            names = [name["name"] for name in projects["result"]]
            name = Prompt.ask("Enter project name", choices=names)
        elif not projects["success"]:
            self.console.print("There was an error fetching your Cloudflare project", style="bold red")
            log.debug(projects)
            return False
        else:
            name = provider.project

        current_version = cf.current_version(name)
        if current_version != self.version:
            log.debug(f"Current Hugo version in Cloudflare: v{current_version}")
        else:
            self.console.print(":heavy_check_mark: Cloudflare - Hugo version is already up to date", style="green bold")
            return False

        response = cf.update_api(name).json()
        if not response["success"]:
            self.console.print("There was an error updating your Cloudflare environment")
            log.debug(response)
            return False
        self.console.print(":heavy_check_mark: Cloudflare", style="green bold")



import os
from typing import Text, Any


class ProviderBase:
    """
    This is a base class for all the providers
    """

    def __init__(self, api_key: Text = None, config_path: Text = None, hugo_version: Text = None):
        """

        :param hugo_version: New Hugo version
        :param api_key: Authentication key for providers
        :param config_path: Path for configuration file of the provider
        """
        self.hugo_version = hugo_version
        self.api_key = api_key
        self.path = config_path

        if not self.api_key:
            self.api_key = os.environ.get("UHUGO_KEY", None)

        if not self.path:
            self.path = os.environ.get("UHUGO_CONFIG_PATH", None)

        if not self.hugo_version:
            self.hugo_version = os.environ.get("HUGO_VERSION", None)

        if not self.api_key and not self.path and not self.hugo_version:
            raise ValueError("api_ky, config_path or hugo_version not set")

    def update_api(self, project_name: Text) -> Any:
        """
        Updates an API

        :param project_name: Name of the project to update
        :return:
        """
        pass

    def update_config_file(self, file_name: Text) -> Any:
        """
        Updates a configuration file

        :return:
        """
        pass

    def get_projects(self, project_name: Text = None) -> Any:
        """
        Checks and gets the current project

        :param project_name: Name of the project to update
        """
        pass

    def current_version(self, project_name: Text = None) -> Any:
        """
        Gets the current Hugo version

        :param project_name: Name of the project to check from
        """
        pass



from importlib.metadata import version
from pathlib import Path

__version__ = version("uhugo")

# Creates a .uhugo folder in user folder
if not Path.home().joinpath(".uhugo").is_dir():
    Path.home().joinpath(".uhugo").mkdir(exist_ok=True)



import logging
import os
from typing import Text, Union, List

import click
from packaging import version
from rich.console import Console
from rich.panel import Panel

from . import __version__
from .checks import check_hugo, get_latest_version_api
from .download import download_hugo_zip
from .install import install_hugo
from .post_install.detect_providers import check_hugo_file, check_providers_fs, Provider
from .post_install.update_providers import UpdateProvider
from .utils import humanise_list

log = logging.getLogger(__name__)
console = Console()


@click.group(name="uhugo", help="uhugo is a Hugo binary helper that downloads and set ups the environment.")
@click.option("--debug", help="Use debug mode", default=False, is_flag=True)
@click.version_option(__version__, package_name="uHugo", prog_name="uHugo")
@click.pass_context
def cli(ctx: click.core.Context, debug: bool):
    ctx.ensure_object(dict)
    ctx.obj["debug"] = debug

    if debug:
        logging.basicConfig(
            level="DEBUG", format="%(asctime)s %(name)s - %(levelname)s:'%(message)s'", datefmt="%d-%b-%y %H:%M:%S"
        )


@cli.command(help="Install latest Hugo binary files")
@click.option("--version", "-v", "ver", default=None, help="Hugo version to download")
@click.option("--force", is_flag=True, default=False, help="Reinstall Hugo")
def install(ver: Text, force: bool):
    hugo = check_hugo()
    if hugo.exists and not force:
        click.echo(console.print("Hugo has already been installed. Use 'uhugo update' to update.", style="red"))
        return

    if force:
        log.debug(f"Deleting existing Hugo at {hugo.path}")
        os.remove(hugo.path)

    with console.status("Fetching latest version", spinner="dots"):
        _ver = get_latest_version_api(ver)
    click.echo(console.print(f"- Latest version is v{_ver}", style="yellow bold"), color=True)

    download_path = download_hugo_zip(_ver)

    with console.status(f"Installing Hugo {_ver}", spinner="dots"):
        installed_path = install_hugo(download_path)

    console.print("\nHugo installed! :tada:\n", style="green bold")

    console.print(Panel.fit(f"[bold green]Make sure {installed_path!r} is in your $PATH", title="Note"))


@cli.command(help="Updates Hugo binary files and any associated configurations")
@click.option("--to", default=None, help="Updates to a specified version")
@click.option("--local", "only_hugo", is_flag=True, help="Updates only local Hugo binary while ignoring providers")
@click.option("--cloud", "only_cloud", is_flag=True, help="Updates only cloud providers while ignoring Hugo")
def update(to: Union[Text, None], only_hugo: bool, only_cloud: bool) -> None:
    hugo = check_hugo()
    if not hugo.exists:
        click.echo(console.print("Hugo is not installed. Use 'uhugo install' to install.", style="red"))
        return

    with console.status("Fetching latest version", spinner="dots"):
        _ver = get_latest_version_api(to)
        log.debug(f"Latest version is {_ver}")

    if (hugo.version >= version.Version(_ver)) and not to and not only_cloud:
        console.print("Hugo is up to date :tada:", style="green")
        return

    if not to and not (hugo.version >= version.Version(_ver)):
        console.print(
            Panel.fit(f"New version available, v{hugo.version} -> v{_ver}", title=f"Hugo v{_ver}"), style="green"
        )

    if not only_cloud:
        download_path = download_hugo_zip(_ver)

        with console.status(f"Installing Hugo {_ver}", spinner="dots"):
            install_hugo(download_path)

        console.print("\nLocal Hugo updated! :tada:\n", style="green bold")

    if only_cloud:
        console.print("Updating only the cloud providers :sun_behind_cloud:\n", style="yellow")

    # ignore cloud provider updates with --hugo flag
    if only_hugo:
        return

    with console.status("Checking providers", spinner="dots") as s:
        providers: List[Provider] = []
        provider = check_hugo_file()
        if provider.name:
            providers.append(provider)
        provider = check_providers_fs()
        providers.extend(provider)

        if not providers:
            return
        console.print(f"{humanise_list([_provider.name for _provider in providers])} found\n")

        s.update("Updating providers")

        try:
            updater = UpdateProvider(providers, console, _ver)
            updater.update()
        except Exception as e:
            log.debug(e)
            s.stop()


def main():
    cli(obj={})
